from ngsolve import *
import netgen.geom2d as geom2d

geo = geom2d.SplineGeometry()
geo.AddRectangle((0, 0), (1,1), bcs=("outer", "outer", "outer", "outer"), leftdomain=1, rightdomain=0)
geo.AddRectangle((0.4, 0.4), (0.6, 0.6), bcs=("inner", "inner", "right", "right"), leftdomain=2, rightdomain=1)
mesh = Mesh(geo.GenerateMesh(maxh=0.5))

# mesh = Mesh(geom2d.unit_square.GenerateMesh(maxh=0.5))

order = 3

# For the Surface->Volume map we need at least order 1!
L = L2(mesh, order=max(1, order))
S = SurfaceL2(mesh, order=order)

## Volume -> Surface
# We can use FacetFESpace as a go-between because it has boundary elements.
F = FacetFESpace(mesh, order=order, dirichlet=".*")
# first L2 -> FacetFESpace
V2S_1 = comp.ConvertOperator(spacea=L, spaceb=F)
# then FacetFESpace -> SurfaceL2
V2S_2 = comp.ConvertOperator(spacea=F, spaceb=S, vb=BND)
V2S = V2S_2 @ V2S_1


## Surface->Volume
# We use a discontiuous H1 space as a go-between. It is discontinuous and has boundary elements.
Hd = Discontinuous(H1(mesh, order=max(1, order)), keep_bnd_els=True)
# first Surface L2 -> discontinuous H1
S2V_1 = comp.ConvertOperator(spacea=S, spaceb=Hd, vb=BND)
# then discontinuous H1 -> L2
S2V_2 = comp.ConvertOperator(spacea=Hd, spaceb=L)
S2V = S2V_2 @ S2V_1

gfL = GridFunction(L)
gfL.Set(0.3 * (1+x)*(2-x) * y)
gfS = GridFunction(S)
gfS.vec.data = V2S * gfL.vec.data
gfL.vec.data = S2V * gfS.vec.data
Draw(gfL, mesh, "gfL")

